/* %Z%%M%	%I% %E% Copyright 1987 J. Schilling */
/*
 *	usefull definitions for dealing with CCS SCSI - devices
 *
 *	Copyright (c) 1987 J. Schilling
 */
/*@@C@@*/

#ifndef	_SCSIREG_H
#define	_SCSIREG_H

#include <utypes.h>
#include <btorder.h>

/* 
 * Sense key values for extended sense.
 */
#define SC_NO_SENSE		0x00
#define SC_RECOVERABLE_ERROR	0x01
#define SC_NOT_READY		0x02
#define SC_MEDIUM_ERROR		0x03
#define SC_HARDWARE_ERROR	0x04
#define SC_ILLEGAL_REQUEST	0x05
#define SC_UNIT_ATTENTION	0x06
#define SC_WRITE_PROTECT	0x07
#define SC_BLANK_CHECK		0x08
#define SC_VENDOR_UNIQUE	0x09
#define SC_COPY_ABORTED		0x0A
#define SC_ABORTED_COMMAND	0x0B
#define SC_EQUAL		0x0C
#define SC_VOLUME_OVERFLOW	0x0D
#define SC_MISCOMPARE		0x0E
#define SC_RESERVED		0x0F

#if	defined(_BIT_FIELDS_LTOH)	/* Intel byteorder */

struct	scsi_inquiry {
	Ucbf	type		: 5;	/*  0 */
	Ucbf	qualifier	: 3;	/*  0 */

	Ucbf	type_modifier	: 7;	/*  1 */
	Ucbf	removable	: 1;	/*  1 */

	Ucbf	ansi_version	: 3;	/*  2 */
	Ucbf	ecma_version	: 3;	/*  2 */
	Ucbf	iso_version	: 2;	/*  2 */

	Ucbf	data_format	: 4;	/*  3 */
	Ucbf	res3_54		: 2;	/*  3 */
	Ucbf	termiop		: 1;	/*  3 */
	Ucbf	aenc		: 1;	/*  3 */

	Ucbf	add_len		: 8;	/*  4 */
	Ucbf	sense_len	: 8;	/*  5 */ /* only Emulex ??? */
	Ucbf	res2		: 8;	/*  6 */

	Ucbf	softreset	: 1;	/*  7 */
	Ucbf	cmdque		: 1;
	Ucbf	res7_2		: 1;
	Ucbf	linked		: 1;
	Ucbf	sync		: 1;
	Ucbf	wbus16		: 1;
	Ucbf	wbus32		: 1;
	Ucbf	reladr		: 1;	/*  7 */

	char	vendor_info[8];		/*  8 */
	char	prod_ident[16];		/* 16 */
	char	prod_revision[4];	/* 32 */
#ifdef	comment
	char	vendor_uniq[20];	/* 36 */
	char	reserved[40];		/* 56 */
#endif
};					/* 96 */

#else					/* Motorola byteorder */

struct	scsi_inquiry {
	Ucbf	qualifier	: 3;	/*  0 */
	Ucbf	type		: 5;	/*  0 */

	Ucbf	removable	: 1;	/*  1 */
	Ucbf	type_modifier	: 7;	/*  1 */

	Ucbf	iso_version	: 2;	/*  2 */
	Ucbf	ecma_version	: 3;
	Ucbf	ansi_version	: 3;	/*  2 */

	Ucbf	aenc		: 1;	/*  3 */
	Ucbf	termiop		: 1;
	Ucbf	res3_54		: 2;
	Ucbf	data_format	: 4;	/*  3 */

	Ucbf	add_len		: 8;	/*  4 */
	Ucbf	sense_len	: 8;	/*  5 */ /* only Emulex ??? */
	Ucbf	res2		: 8;	/*  6 */
	Ucbf	reladr		: 1;	/*  7 */
	Ucbf	wbus32		: 1;
	Ucbf	wbus16		: 1;
	Ucbf	sync		: 1;
	Ucbf	linked		: 1;
	Ucbf	res7_2		: 1;
	Ucbf	cmdque		: 1;
	Ucbf	softreset	: 1;
	char	vendor_info[8];		/*  8 */
	char	prod_ident[16];		/* 16 */
	char	prod_revision[4];	/* 32 */
#ifdef	comment
	char	vendor_uniq[20];	/* 36 */
	char	reserved[40];		/* 56 */
#endif
};					/* 96 */
#endif

#define	info		vendor_info
#define	ident		prod_ident
#define	revision	prod_revision

/* Peripheral Device Qualifier */

#define	INQ_DEV_PRESENT	0x00		/* Physical device present */
#define	INQ_DEV_NOTPR	0x01		/* Physical device not present */
#define	INQ_DEV_RES	0x02		/* Reserved */
#define	INQ_DEV_NOTSUP	0x03		/* Logical unit not supported */

/* Peripheral Device Type */

#define	INQ_DASD	0x00		/* Direct-access device (disk) */
#define	INQ_SEQD	0x01		/* Sequential-access device (tape) */
#define	INQ_PRTD	0x02 		/* Printer device */
#define	INQ_PROCD	0x03 		/* Processor device */
#define	INQ_OPTD	0x04		/* Write once device (optical disk) */
#define	INQ_WORM	0x04		/* Write once device (optical disk) */
#define	INQ_ROMD	0x05		/* CD-ROM device */
#define	INQ_SCAN	0x06		/* Scanner device */
#define	INQ_OMEM	0x07		/* Optical Memory device */
#define	INQ_JUKE	0x08		/* Medium Changer device (jukebox) */
#define	INQ_COMM	0x09		/* Communications device */
#define	INQ_IT8_1	0x0A		/* IT8 */
#define	INQ_IT8_2	0x0B		/* IT8 */
#define	INQ_STARR	0x0C		/* Storage array device */
#define	INQ_ENCL	0x0D		/* Enclosure services device */
#define	INQ_NODEV	0x1F		/* Unknown or no device */
#define	INQ_NOTPR	0x1F		/* Logical unit not present (SCSI-1) */

#if	defined(_BIT_FIELDS_LTOH)	/* Intel byteorder */

struct scsi_mode_header {
	Ucbf	sense_data_len	: 8;
	u_char	medium_type;
	Ucbf	res2		: 4;
	Ucbf	cache		: 1;
	Ucbf	res		: 2;
	Ucbf	write_prot	: 1;
	u_char	blockdesc_len;
};

#else					/* Motorola byteorder */

struct scsi_mode_header {
	Ucbf	sense_data_len	: 8;
	u_char	medium_type;
	Ucbf	write_prot	: 1;
	Ucbf	res		: 2;
	Ucbf	cache		: 1;
	Ucbf	res2		: 4;
	u_char	blockdesc_len;
};
#endif

struct scsi_modesel_header {
	Ucbf	sense_data_len	: 8;
	u_char	medium_type;
	Ucbf	res2		: 8;
	u_char	blockdesc_len;
};

struct scsi_mode_blockdesc {
	u_char	density;
	u_char	nlblock[3];
	Ucbf	res		: 8;
	u_char	lblen[3];
};

#if	defined(_BIT_FIELDS_LTOH)	/* Intel byteorder */

struct acb_mode_data {
	u_char	listformat;
	u_char	ncyl[2];
	u_char	nhead;
	u_char	start_red_wcurrent[2];
	u_char	start_precomp[2];
	u_char	landing_zone;
	u_char	step_rate;
	Ucbf			: 2;
	Ucbf	hard_sec	: 1;
	Ucbf	fixed_media	: 1;
	Ucbf			: 4;
	u_char	sect_per_trk;
};

#else					/* Motorola byteorder */

struct acb_mode_data {
	u_char	listformat;
	u_char	ncyl[2];
	u_char	nhead;
	u_char	start_red_wcurrent[2];
	u_char	start_precomp[2];
	u_char	landing_zone;
	u_char	step_rate;
	Ucbf			: 4;
	Ucbf	fixed_media	: 1;
	Ucbf	hard_sec	: 1;
	Ucbf			: 2;
	u_char	sect_per_trk;
};
#endif

#if	defined(_BIT_FIELDS_LTOH)	/* Intel byteorder */

struct scsi_mode_page_header {
	Ucbf	p_code		: 6;
	Ucbf	res		: 1;
	Ucbf	parsave		: 1;
	u_char	p_len;
};

/*
 * This is a hack that allows mode pages without
 * any further bitfileds to be defined bitorder independent.
 */
#define	MP_P_CODE			\
	Ucbf	p_code		: 6;	\
	Ucbf	p_res		: 1;	\
	Ucbf	parsave		: 1

#else					/* Motorola byteorder */

struct scsi_mode_page_header {
	Ucbf	parsave		: 1;
	Ucbf	res		: 1;
	Ucbf	p_code		: 6;
	u_char	p_len;
};

/*
 * This is a hack that allows mode pages without
 * any further bitfileds to be defined bitorder independent.
 */
#define	MP_P_CODE			\
	Ucbf	parsave		: 1;	\
	Ucbf	p_res		: 1;	\
	Ucbf	p_code		: 6

#endif

#if	defined(_BIT_FIELDS_LTOH)	/* Intel byteorder */

struct scsi_mode_page_01 {		/* Error recovery Parameters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x0A = 12 Bytes */
	Ucbf	disa_correction	: 1;	/* Byte 2 */
	Ucbf	term_on_rec_err	: 1;
	Ucbf	report_rec_err	: 1;
	Ucbf	en_early_corr	: 1;
	Ucbf	read_continuous	: 1;
	Ucbf	tranfer_block	: 1;
	Ucbf	en_auto_reall_r	: 1;
	Ucbf	en_auto_reall_w	: 1;	/* Byte 2 */
	u_char	rd_retry_count;		/* Byte 3 */
	u_char	correction_span;
	char	head_offset_count;
	char	data_strobe_offset;
	u_char	res;
	u_char	wr_retry_count;
	u_char	res_tape[2];
	u_char	recov_timelim[2];
};

#else					/* Motorola byteorder */

struct scsi_mode_page_01 {		/* Error recovery Parameters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x0A = 12 Bytes */
	Ucbf	en_auto_reall_w	: 1;	/* Byte 2 */
	Ucbf	en_auto_reall_r	: 1;
	Ucbf	tranfer_block	: 1;
	Ucbf	read_continuous	: 1;
	Ucbf	en_early_corr	: 1;
	Ucbf	report_rec_err	: 1;
	Ucbf	term_on_rec_err	: 1;
	Ucbf	disa_correction	: 1;	/* Byte 2 */
	u_char	rd_retry_count;		/* Byte 3 */
	u_char	correction_span;
	char	head_offset_count;
	char	data_strobe_offset;
	u_char	res;
	u_char	wr_retry_count;
	u_char	res_tape[2];
	u_char	recov_timelim[2];
};
#endif


#if	defined(_BIT_FIELDS_LTOH)	/* Intel byteorder */

struct scsi_mode_page_02 {		/* Device dis/re connect Parameters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x0E = 16 Bytes */
	u_char	buf_full_ratio;
	u_char	buf_empt_ratio;
	u_char	bus_inact_limit[2];
	u_char	disc_time_limit[2];
	u_char	conn_time_limit[2];
	u_char	max_burst_size[2];	/* Start SCSI-2 */
	Ucbf	data_tr_dis_ctl	: 2;
	Ucbf			: 6;
	u_char	res[3];
};

#else					/* Motorola byteorder */

struct scsi_mode_page_02 {		/* Device dis/re connect Parameters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x0E = 16 Bytes */
	u_char	buf_full_ratio;
	u_char	buf_empt_ratio;
	u_char	bus_inact_limit[2];
	u_char	disc_time_limit[2];
	u_char	conn_time_limit[2];
	u_char	max_burst_size[2];	/* Start SCSI-2 */
	Ucbf			: 6;
	Ucbf	data_tr_dis_ctl	: 2;
	u_char	res[3];
};
#endif

#define	DTDC_DATADONE	0x01		/*
					 * Target may not disconnect once
					 * data transfer is started until
					 * all data successfully transferred.
					 */

#define	DTDC_CMDDONE	0x03		/*
					 * Target may not disconnect once
					 * data transfer is started until
					 * command completed.
					 */


#if	defined(_BIT_FIELDS_LTOH)	/* Intel byteorder */

struct scsi_mode_page_03 {		/* Direct access format Paramters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x16 = 24 Bytes */
	u_char	trk_per_zone[2];
	u_char	alt_sec_per_zone[2];
	u_char	alt_trk_per_zone[2];
	u_char	alt_trk_per_vol[2];
	u_char	sect_per_trk[2];
	u_char	bytes_per_phys_sect[2];
	u_char	interleave[2];
	u_char	trk_skew[2];
	u_char	cyl_skew[2];
	Ucbf			: 3;
	Ucbf	inhibit_save	: 1;
	Ucbf	fmt_by_surface	: 1;
	Ucbf	removable	: 1;
	Ucbf	hard_sec	: 1;
	Ucbf	soft_sec	: 1;
	u_char	res[3];
};

#else					/* Motorola byteorder */

struct scsi_mode_page_03 {		/* Direct access format Paramters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x16 = 24 Bytes */
	u_char	trk_per_zone[2];
	u_char	alt_sec_per_zone[2];
	u_char	alt_trk_per_zone[2];
	u_char	alt_trk_per_vol[2];
	u_char	sect_per_trk[2];
	u_char	bytes_per_phys_sect[2];
	u_char	interleave[2];
	u_char	trk_skew[2];
	u_char	cyl_skew[2];
	Ucbf	soft_sec	: 1;
	Ucbf	hard_sec	: 1;
	Ucbf	removable	: 1;
	Ucbf	fmt_by_surface	: 1;
	Ucbf	inhibit_save	: 1;
	Ucbf			: 3;
	u_char	res[3];
};
#endif

#if	defined(_BIT_FIELDS_LTOH)	/* Intel byteorder */

struct scsi_mode_page_04 {		/* Rigid disk Geometry Parameters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x16 = 24 Bytes */
	u_char	ncyl[3];
	u_char	nhead;
	u_char	start_precomp[3];
	u_char	start_red_wcurrent[3];
	u_char	step_rate[2];
	u_char	landing_zone[3];
	Ucbf	rot_pos_locking	: 2;	/* Start SCSI-2 */
	Ucbf			: 6;	/* Start SCSI-2 */
	u_char	rotational_off;
	u_char	res1;
	u_char	rotation_rate[2];
	u_char	res2[2];
};

#else					/* Motorola byteorder */

struct scsi_mode_page_04 {		/* Rigid disk Geometry Parameters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x16 = 24 Bytes */
	u_char	ncyl[3];
	u_char	nhead;
	u_char	start_precomp[3];
	u_char	start_red_wcurrent[3];
	u_char	step_rate[2];
	u_char	landing_zone[3];
	Ucbf			: 6;	/* Start SCSI-2 */
	Ucbf	rot_pos_locking	: 2;	/* Start SCSI-2 */
	u_char	rotational_off;
	u_char	res1;
	u_char	rotation_rate[2];
	u_char	res2[2];
};
#endif

#if	defined(_BIT_FIELDS_LTOH)	/* Intel byteorder */

struct scsi_mode_page_05 {		/* Flexible disk Parameters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x1E = 32 Bytes */
	u_char	transfer_rate[2];
	u_char	nhead;
	u_char	sect_per_trk;
	u_char	bytes_per_phys_sect[2];
	u_char	ncyl[2];
	u_char	start_precomp[2];
	u_char	start_red_wcurrent[2];
	u_char	step_rate[2];
	u_char	step_pulse_width;
	u_char	head_settle_delay[2];
	u_char	motor_on_delay;
	u_char	motor_off_delay;
	Ucbf	spc		: 4;
	Ucbf			: 4;
	Ucbf			: 5;
	Ucbf	mo		: 1;
	Ucbf	ssn		: 1;
	Ucbf	trdy		: 1;
	u_char	write_compensation;
	u_char	head_load_delay;
	u_char	head_unload_delay;
	Ucbf	pin_2_use	: 4;
	Ucbf	pin_34_use	: 4;
	Ucbf	pin_1_use	: 4;
	Ucbf	pin_4_use	: 4;
	u_char	rotation_rate[2];
	u_char	res[2];
};

#else					/* Motorola byteorder */

struct scsi_mode_page_05 {		/* Flexible disk Parameters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x1E = 32 Bytes */
	u_char	transfer_rate[2];
	u_char	nhead;
	u_char	sect_per_trk;
	u_char	bytes_per_phys_sect[2];
	u_char	ncyl[2];
	u_char	start_precomp[2];
	u_char	start_red_wcurrent[2];
	u_char	step_rate[2];
	u_char	step_pulse_width;
	u_char	head_settle_delay[2];
	u_char	motor_on_delay;
	u_char	motor_off_delay;
	Ucbf	trdy		: 1;
	Ucbf	ssn		: 1;
	Ucbf	mo		: 1;
	Ucbf			: 5;
	Ucbf			: 4;
	Ucbf	spc		: 4;
	u_char	write_compensation;
	u_char	head_load_delay;
	u_char	head_unload_delay;
	Ucbf	pin_34_use	: 4;
	Ucbf	pin_2_use	: 4;
	Ucbf	pin_4_use	: 4;
	Ucbf	pin_1_use	: 4;
	u_char	rotation_rate[2];
	u_char	res[2];
};
#endif

#if	defined(_BIT_FIELDS_LTOH)	/* Intel byteorder */

struct scsi_mode_page_07 {		/* Verify Error recovery */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x0A = 12 Bytes */
	Ucbf	disa_correction	: 1;	/* Byte 2 */
	Ucbf	term_on_rec_err	: 1;
	Ucbf	report_rec_err	: 1;
	Ucbf	en_early_corr	: 1;
	Ucbf	res		: 4;	/* Byte 2 */
	u_char	ve_retry_count;		/* Byte 3 */
	u_char	ve_correction_span;
	char	res2[5];		/* Byte 5 */
	u_char	ve_recov_timelim[2];	/* Byte 10 */
};

#else					/* Motorola byteorder */

struct scsi_mode_page_07 {		/* Verify Error recovery */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x0A = 12 Bytes */
	Ucbf	res		: 4;	/* Byte 2 */
	Ucbf	en_early_corr	: 1;
	Ucbf	report_rec_err	: 1;
	Ucbf	term_on_rec_err	: 1;
	Ucbf	disa_correction	: 1;	/* Byte 2 */
	u_char	ve_retry_count;		/* Byte 3 */
	u_char	ve_correction_span;
	char	res2[5];		/* Byte 5 */
	u_char	ve_recov_timelim[2];	/* Byte 10 */
};
#endif

#if	defined(_BIT_FIELDS_LTOH)	/* Intel byteorder */

struct scsi_mode_page_08 {		/* Caching Parameters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x0A = 12 Bytes */
	Ucbf	disa_rd_cache	: 1;	/* Byte 2 */
	Ucbf	muliple_fact	: 1;
	Ucbf	en_wt_cache	: 1;
	Ucbf	res		: 5;	/* Byte 2 */
	Ucbf	wt_ret_pri	: 4;	/* Byte 3 */
	Ucbf	demand_rd_ret_pri: 4;	/* Byte 3 */
	u_char	disa_pref_tr_len[2];	/* Byte 4 */
	u_char	min_pref[2];		/* Byte 6 */
	u_char	max_pref[2];		/* Byte 8 */
	u_char	max_pref_ceiling[2];	/* Byte 10 */
};

#else					/* Motorola byteorder */

struct scsi_mode_page_08 {		/* Caching Parameters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x0A = 12 Bytes */
	Ucbf	res		: 5;	/* Byte 2 */
	Ucbf	en_wt_cache	: 1;
	Ucbf	muliple_fact	: 1;
	Ucbf	disa_rd_cache	: 1;	/* Byte 2 */
	Ucbf	demand_rd_ret_pri: 4;	/* Byte 3 */
	Ucbf	wt_ret_pri	: 4;
	u_char	disa_pref_tr_len[2];	/* Byte 4 */
	u_char	min_pref[2];		/* Byte 6 */
	u_char	max_pref[2];		/* Byte 8 */
	u_char	max_pref_ceiling[2];	/* Byte 10 */
};
#endif

struct scsi_mode_page_09 {		/* Peripheral device Parameters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* >= 0x06 = 8 Bytes */
	u_char	interface_id[2];	/* Byte 2 */
	u_char	res[4];			/* Byte 4 */
	u_char	vendor_specific[1];	/* Byte 8 */
};

#define	PDEV_SCSI	0x0000		/* scsi interface */
#define	PDEV_SMD	0x0001		/* SMD interface */
#define	PDEV_ESDI	0x0002		/* ESDI interface */
#define	PDEV_IPI2	0x0003		/* IPI-2 interface */
#define	PDEV_IPI3	0x0004		/* IPI-3 interface */

#if	defined(_BIT_FIELDS_LTOH)	/* Intel byteorder */

struct scsi_mode_page_0A {		/* Common device Control Parameters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x06 = 8 Bytes */
	Ucbf	rep_log_exeption: 1;	/* Byte 2 */
	Ucbf	res		: 7;	/* Byte 2 */
	Ucbf	dis_queuing	: 1;	/* Byte 3 */
	Ucbf	queuing_err_man	: 1;
	Ucbf	res2		: 2;
	Ucbf	queue_alg_mod	: 4;	/* Byte 3 */
	Ucbf	EAENP		: 1;	/* Byte 4 */
	Ucbf	UAENP		: 1;
	Ucbf	RAENP		: 1;
	Ucbf	res3		: 4;
	Ucbf	en_ext_cont_all	: 1;	/* Byte 4 */
	Ucbf	res4		: 8;
	u_char	ready_aen_hold_per[2];	/* Byte 6 */
};

#else					/* Motorola byteorder */

struct scsi_mode_page_0A {		/* Common device Control Parameters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x06 = 8 Bytes */
	Ucbf	res		: 7;	/* Byte 2 */
	Ucbf	rep_log_exeption: 1;	/* Byte 2 */
	Ucbf	queue_alg_mod	: 4;	/* Byte 3 */
	Ucbf	res2		: 2;
	Ucbf	queuing_err_man	: 1;
	Ucbf	dis_queuing	: 1;	/* Byte 3 */
	Ucbf	en_ext_cont_all	: 1;	/* Byte 4 */
	Ucbf	res3		: 4;
	Ucbf	RAENP		: 1;
	Ucbf	UAENP		: 1;
	Ucbf	EAENP		: 1;	/* Byte 4 */
	Ucbf	res4		: 8;
	u_char	ready_aen_hold_per[2];	/* Byte 6 */
};
#endif

#define	CTRL_QMOD_RESTRICT	0x0
#define	CTRL_QMOD_UNRESTRICT	0x1


struct scsi_mode_page_0B {		/* Medium Types Supported Parameters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x06 = 8 Bytes */
	u_char	res[2];			/* Byte 2 */
	u_char	medium_one_supp;	/* Byte 4 */
	u_char	medium_two_supp;	/* Byte 5 */
	u_char	medium_three_supp;	/* Byte 6 */
	u_char	medium_four_supp;	/* Byte 7 */
};

#if	defined(_BIT_FIELDS_LTOH)	/* Intel byteorder */

struct scsi_mode_page_0C {		/* Notch & Partition Parameters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x16 = 24 Bytes */
	Ucbf	res		: 6;	/* Byte 2 */
	Ucbf	logical_notch	: 1;
	Ucbf	notched_drive	: 1;	/* Byte 2 */
	u_char	res2;			/* Byte 3 */
	u_char	max_notches[2];		/* Byte 4  */
	u_char	active_notch[2];	/* Byte 6  */
	u_char	starting_boundary[4];	/* Byte 8  */
	u_char	ending_boundary[4];	/* Byte 12 */
	u_char	pages_notched[8];	/* Byte 16 */
};

#else					/* Motorola byteorder */

struct scsi_mode_page_0C {		/* Notch & Partition Parameters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x16 = 24 Bytes */
	Ucbf	notched_drive	: 1;	/* Byte 2 */
	Ucbf	logical_notch	: 1;
	Ucbf	res		: 6;	/* Byte 2 */
	u_char	res2;			/* Byte 3 */
	u_char	max_notches[2];		/* Byte 4  */
	u_char	active_notch[2];	/* Byte 6  */
	u_char	starting_boundary[4];	/* Byte 8  */
	u_char	ending_boundary[4];	/* Byte 12 */
	u_char	pages_notched[8];	/* Byte 16 */
};
#endif

#if	defined(_BIT_FIELDS_LTOH)	/* Intel byteorder */

struct scsi_mode_page_0D {		/* CD-ROM Parameters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x06 = 8 Bytes */
	u_char	res;			/* Byte 2 */
	Ucbf	inact_timer_mult: 4;	/* Byte 3 */
	Ucbf	res2		: 4;	/* Byte 3 */
	u_char	s_un_per_m_un[2];	/* Byte 4  */
	u_char	f_un_per_s_un[2];	/* Byte 6  */
};

#else					/* Motorola byteorder */

struct scsi_mode_page_0D {		/* CD-ROM Parameters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x06 = 8 Bytes */
	u_char	res;			/* Byte 2 */
	Ucbf	res2		: 4;	/* Byte 3 */
	Ucbf	inact_timer_mult: 4;	/* Byte 3 */
	u_char	s_un_per_m_un[2];	/* Byte 4  */
	u_char	f_un_per_s_un[2];	/* Byte 6  */
};
#endif

struct sony_mode_page_20 {		/* Sony Format Mode Parameters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x0A = 12 Bytes */
	u_char	format_mode;
	u_char	format_type;
#define	num_bands	user_band_size	/* Gilt bei Type 1 */
	u_char	user_band_size[4];	/* Gilt bei Type 0 */
	u_char	spare_band_size[2];
	u_char	res[2];
};

#if	defined(_BIT_FIELDS_LTOH)	/* Intel byteorder */

struct toshiba_mode_page_20 {		/* Toshiba Speed Control Parameters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x01 = 3 Bytes */
	Ucbf	speed		: 1;
	Ucbf	res		: 7;
};

#else					/* Motorola byteorder */

struct toshiba_mode_page_20 {		/* Toshiba Speed Control Parameters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x01 = 3 Bytes */
	Ucbf	res		: 7;
	Ucbf	speed		: 1;
};
#endif

#if	defined(_BIT_FIELDS_LTOH)	/* Intel byteorder */

struct ccs_mode_page_38 {		/* CCS Caching Parameters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x0E = 14 Bytes */

	Ucbf	cache_table_size: 4;	/* Byte 3 */
	Ucbf	cache_en	: 1;
	Ucbf	res2		: 1;
	Ucbf	wr_index_en	: 1;
	Ucbf	res		: 1;	/* Byte 3 */
	u_char	threshold;		/* Byte 4 Prefetch threshold */
	u_char	max_prefetch;		/* Byte 5 Max. prefetch */
	u_char	max_multiplier;		/* Byte 6 Max. prefetch multiplier */
	u_char	min_prefetch;		/* Byte 7 Min. prefetch */
	u_char	min_multiplier;		/* Byte 8 Min. prefetch multiplier */
	u_char	res3[8];		/* Byte 9 */
};

#else					/* Motorola byteorder */

struct ccs_mode_page_38 {		/* CCS Caching Parameters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x0E = 14 Bytes */

	Ucbf	res		: 1;	/* Byte 3 */
	Ucbf	wr_index_en	: 1;
	Ucbf	res2		: 1;
	Ucbf	cache_en	: 1;
	Ucbf	cache_table_size: 4;	/* Byte 3 */
	u_char	threshold;		/* Byte 4 Prefetch threshold */
	u_char	max_prefetch;		/* Byte 5 Max. prefetch */
	u_char	max_multiplier;		/* Byte 6 Max. prefetch multiplier */
	u_char	min_prefetch;		/* Byte 7 Min. prefetch */
	u_char	min_multiplier;		/* Byte 8 Min. prefetch multiplier */
	u_char	res3[8];		/* Byte 9 */
};
#endif

#if defined(_BIT_FIELDS_LTOH)	/* Intel byteorder */

struct cd_mode_page_05 {		/* write parameters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x32 = 50 Bytes */
	Ucbf	write_type	:4;	/* Session write type (PACKET/TAO...)*/
	Ucbf	test_write	:1;	/* Do not actually write data	     */
	Ucbf	res_2		:3;
	Ucbf	track_mode	:4;	/* Track mode (Q-sub control nibble) */
	Ucbf	copy		:1;	/* 1st higher gen of copy prot track ~*/
	Ucbf	fp		:1;	/* Fixed packed (if in packet mode)  */
	Ucbf	multi_session	:2;	/* Multi session write type	     */
	Ucbf	dbtype		:4;	/* Data block type		     */
	Ucbf	res_4		:4;	/* Reserved			     */
	u_char	res_56[2];		/* Reserved			     */
	Ucbf	host_appl_code	:6;	/* Host application code of disk     */
	Ucbf	res_7		:2;	/* Reserved			     */
	u_char	session_format;		/* Session format (DA/CDI/XA)	     */
	u_char	res_9;			/* Reserved			     */
	u_char	packet_size[4];		/* # of user datablocks/fixed packet */
	u_char	audio_pause_len[2];	/* # of blocks where index is zero   */
	u_char	media_cat_number[16];	/* Media catalog Number (MCN)	     */
	u_char	ISRC[14];		/* ISRC for this track		     */
	u_char	sub_header[4];
	u_char	vendor_uniq[4];
};

#else				/* Motorola byteorder */

struct cd_mode_page_05 {		/* write parameters */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x32 = 50 Bytes */
	Ucbf	res_2		:3;
	Ucbf	test_write	:1;	/* Do not actually write data	     */
	Ucbf	write_type	:4;	/* Session write type (PACKET/TAO...)*/
	Ucbf	multi_session	:2;	/* Multi session write type	     */
	Ucbf	fp		:1;	/* Fixed packed (if in packet mode)  */
	Ucbf	copy		:1;	/* 1st higher gen of copy prot track */
	Ucbf	track_mode	:4;	/* Track mode (Q-sub control nibble) */
	Ucbf	res_4		:4;	/* Reserved			     */
	Ucbf	dbtype		:4;	/* Data block type		     */
	u_char	res_56[2];		/* Reserved			     */
	Ucbf	res_7		:2;	/* Reserved			     */
	Ucbf	host_appl_code	:6;	/* Host application code of disk     */
	u_char	session_format;		/* Session format (DA/CDI/XA)	     */
	u_char	res_9;			/* Reserved			     */
	u_char	packet_size[4];		/* # of user datablocks/fixed packet */
	u_char	audio_pause_len[2];	/* # of blocks where index is zero   */
	u_char	media_cat_number[16];	/* Media catalog Number (MCN)	     */
	u_char	ISRC[14];		/* ISRC for this track		     */
	u_char	sub_header[4];
	u_char	vendor_uniq[4];
};

#endif

#if defined(_BIT_FIELDS_LTOH)	/* Intel byteorder */

struct cd_mode_page_2A {		/* CD Cap / mech status */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x14 = 20 Bytes */
	Ucbf	cd_r_read	:1;	/* Reads CD-R  media		     */
	Ucbf	cd_rw_read	:1;	/* Reads CD-RW media		     */
	Ucbf	method2		:1;	/* Reads fixed packet method2 media  */
	Ucbf	res_2		:5;
	Ucbf	cd_r_write	:1;	/* Supports writing CD-R  media	     */
	Ucbf	cd_rw_write	:1;	/* Supports writing CD-RW media	     */
	Ucbf	test_write	:1;	/* Supports emulation write	     */
	Ucbf	res_3		:5;	/* Reseved			     */
	Ucbf	audio_play	:1;	/* Supports Audio play operation     */
	Ucbf	composite	:1;	/* Deliveres composite A/V stream    */
	Ucbf	digital_port_2	:1;	/* Supports digital output on port 2 */
	Ucbf	digital_port_1	:1;	/* Supports digital output on port 1 */
	Ucbf	mode_2_form_1	:1;	/* Reads Mode-2 form 1 media (XA)    */
	Ucbf	mode_2_form_2	:1;	/* Reads Mode-2 form 2 media	     */
	Ucbf	multi_session	:1;	/* Reads multi-session media	     */
	Ucbf	res_4		:1;	/* Reserved			     */
	Ucbf	cd_da_supported	:1;	/* Reads audio data with READ CD cmd */
	Ucbf	cd_da_accurate	:1;	/* READ CD data stream is accurate   */
	Ucbf	rw_supported	:1;	/* Reads R-W sub channel information */
	Ucbf	rw_deint_corr	:1;	/* Reads de-interleved R-W sub chan  */
	Ucbf	c2_pointers	:1;	/* Supports C2 error pointers	     */
	Ucbf	ISRC		:1;	/* Reads ISRC information	     */
	Ucbf	UPC		:1;	/* Reads media catalog number (UPC)  */
	Ucbf	read_bar_code	:1;	/* Supports reading bar codes	     */
	Ucbf	lock		:1;	/* PREVENT/ALLOW may lock media	     */
	Ucbf	lock_state	:1;	/* Lock state 0=unlocked 1=locked    */
	Ucbf	prevent_jumper	:1;	/* State of prev/allow jumper 0=pres */
	Ucbf	eject		:1;	/* Ejects disc/cartr with STOP LoEj  */
	Ucbf	res_6_4		:1;	/* Reserved			     */
	Ucbf	loading_type	:3;	/* Loading mechanism type	     */
	Ucbf	sep_chan_vol	:1;	/* Vol controls each channel separat */
	Ucbf	sep_chan_mute	:1;	/* Mute controls each channel separat*/
	Ucbf	disk_present_rep:1;	/* Changer supports disk present rep */
	Ucbf	sw_slot_sel	:1;	/* Load empty slot in changer	     */
	Ucbf	res_7		:4;	/* Reserved			     */
	u_char	max_read_speed[2];	/* Max. read speed in KB/s	     */
	u_char	num_vol_levels[2];	/* # of supported volume levels	     */
	u_char	buffer_size[2];		/* Buffer size for the data in KB    */
	u_char	cur_read_speed[2];	/* Current read speed in KB/s	     */
	u_char	res_16;			/* Reserved			     */
	Ucbf	res_17_0	:1;	/* Reserved			     */
	Ucbf	BCK		:1;	/* Data valid on falling edge of BCK */
	Ucbf	RCK		:1;	/* Set: HIGH high LRCK=left channel  */
	Ucbf	LSBF		:1;	/* Set: LSB first Clear: MSB first   */
	Ucbf	length		:2;	/* 0=32BCKs 1=16BCKs 2=24BCKs 3=24I2c*/
	Ucbf	res_17		:2;	/* Reserved			     */
	u_char	max_write_speed[2];	/* Max. write speed supported in KB/s*/
	u_char	cur_write_speed[2];	/* Current write speed in KB/s	     */
};

#else				/* Motorola byteorder */

struct cd_mode_page_2A {		/* CD Cap / mech status */
		MP_P_CODE;		/* parsave & pagecode */
	u_char	p_len;			/* 0x14 = 20 Bytes */
	Ucbf	res_2		:5;
	Ucbf	method2		:1;	/* Reads fixed packet method2 media  */
	Ucbf	cd_rw_read	:1;	/* Reads CD-RW media		     */
	Ucbf	cd_r_read	:1;	/* Reads CD-R  media		     */
	Ucbf	res_3		:5;	/* Reseved			     */
	Ucbf	test_write	:1;	/* Supports emulation write	     */
	Ucbf	cd_rw_write	:1;	/* Supports writing CD-RW media	     */
	Ucbf	cd_r_write	:1;	/* Supports writing CD-R  media	     */
	Ucbf	res_4		:1;	/* Reserved			     */
	Ucbf	multi_session	:1;	/* Reads multi-session media	     */
	Ucbf	mode_2_form_2	:1;	/* Reads Mode-2 form 2 media	     */
	Ucbf	mode_2_form_1	:1;	/* Reads Mode-2 form 1 media (XA)    */
	Ucbf	digital_port_1	:1;	/* Supports digital output on port 1 */
	Ucbf	digital_port_2	:1;	/* Supports digital output on port 2 */
	Ucbf	composite	:1;	/* Deliveres composite A/V stream    */
	Ucbf	audio_play	:1;	/* Supports Audio play operation     */
	Ucbf	read_bar_code	:1;	/* Supports reading bar codes	     */
	Ucbf	UPC		:1;	/* Reads media catalog number (UPC)  */
	Ucbf	ISRC		:1;	/* Reads ISRC information	     */
	Ucbf	c2_pointers	:1;	/* Supports C2 error pointers	     */
	Ucbf	rw_deint_corr	:1;	/* Reads de-interleved R-W sub chan  */
	Ucbf	rw_supported	:1;	/* Reads R-W sub channel information */
	Ucbf	cd_da_accurate	:1;	/* READ CD data stream is accurate   */
	Ucbf	cd_da_supported	:1;	/* Reads audio data with READ CD cmd */
	Ucbf	loading_type	:3;	/* Loading mechanism type	     */
	Ucbf	res_6_4		:1;	/* Reserved			     */
	Ucbf	eject		:1;	/* Ejects disc/cartr with STOP LoEj  */
	Ucbf	prevent_jumper	:1;	/* State of prev/allow jumper 0=pres */
	Ucbf	lock_state	:1;	/* Lock state 0=unlocked 1=locked    */
	Ucbf	lock		:1;	/* PREVENT/ALLOW may lock media	     */
	Ucbf	res_7		:4;	/* Reserved			     */
	Ucbf	sw_slot_sel	:1;	/* Load empty slot in changer	     */
	Ucbf	disk_present_rep:1;	/* Changer supports disk present rep */
	Ucbf	sep_chan_mute	:1;	/* Mute controls each channel separat*/
	Ucbf	sep_chan_vol	:1;	/* Vol controls each channel separat */
	u_char	max_read_speed[2];	/* Max. read speed in KB/s	     */
	u_char	num_vol_levels[2];	/* # of supported volume levels	     */
	u_char	buffer_size[2];		/* Buffer size for the data in KB    */
	u_char	cur_read_speed[2];	/* Current read speed in KB/s	     */
	u_char	res_16;			/* Reserved			     */
	Ucbf	res_17		:2;	/* Reserved			     */
	Ucbf	length		:2;	/* 0=32BCKs 1=16BCKs 2=24BCKs 3=24I2c*/
	Ucbf	LSBF		:1;	/* Set: LSB first Clear: MSB first   */
	Ucbf	RCK		:1;	/* Set: HIGH high LRCK=left channel  */
	Ucbf	BCK		:1;	/* Data valid on falling edge of BCK */
	Ucbf	res_17_0	:1;	/* Reserved			     */
	u_char	max_write_speed[2];	/* Max. write speed supported in KB/s*/
	u_char	cur_write_speed[2];	/* Current write speed in KB/s	     */
};

#endif

#define	LT_CADDY	0
#define	LT_TRAY		1
#define	LT_POP_UP	2
#define	LT_RES3		3
#define	LT_CHANGER_IND	4
#define	LT_CHANGER_CART	5
#define	LT_RES6		6
#define	LT_RES7		7


struct scsi_mode_data {
	struct scsi_mode_header		header;
	struct scsi_mode_blockdesc	blockdesc;
	union	pagex	{
		struct acb_mode_data		acb;
		struct scsi_mode_page_01	page1;
		struct scsi_mode_page_02	page2;
		struct scsi_mode_page_03	page3;
		struct scsi_mode_page_04	page4;
		struct scsi_mode_page_05	page5;
		struct scsi_mode_page_07	page7;
		struct scsi_mode_page_08	page8;
		struct scsi_mode_page_09	page9;
		struct scsi_mode_page_0A	pageA;
		struct scsi_mode_page_0B	pageB;
		struct scsi_mode_page_0C	pageC;
		struct scsi_mode_page_0D	pageD;
		struct sony_mode_page_20	sony20;
		struct toshiba_mode_page_20	toshiba20;
		struct ccs_mode_page_38		ccs38;
	} pagex;
};

struct scsi_capacity {
	long	c_baddr;		/* must convert byteorder!! */
	long	c_bsize;		/* must convert byteorder!! */
};

#if	defined(_BIT_FIELDS_LTOH)	/* Intel byteorder */

struct scsi_def_header {
	Ucbf		:8;
	Ucbf	format	:3;
	Ucbf	gdl	:1;
	Ucbf	mdl	:1;
	Ucbf		:3;
	u_char	length[2];
};

#else					/* Motorola byteorder */

struct scsi_def_header {
	Ucbf		:8;
	Ucbf		:3;
	Ucbf	mdl	:1;
	Ucbf	gdl	:1;
	Ucbf	format	:3;
	u_char	length[2];
};
#endif


#if	defined(_BIT_FIELDS_LTOH)	/* Intel byteorder */

struct scsi_format_header {
	Ucbf	res		: 8;	/* Adaptec 5500: 1 --> format track */
	Ucbf	vu		: 1;
	Ucbf			: 3;
	Ucbf	serr		: 1;	/* Stop on error		    */
	Ucbf	dcert		: 1;	/* Disable certification	    */
	Ucbf	dmdl		: 1;	/* Disable manufacturer defect list */
	Ucbf	enable		: 1;	/* Enable to use the next 3 bits    */
	u_char	length[2];		/* Length of following list in bytes*/
};

#else					/* Motorola byteorder */

struct scsi_format_header {
	Ucbf	res		: 8;	/* Adaptec 5500: 1 --> format track */
	Ucbf	enable		: 1;	/* Enable to use the next 3 bits    */
	Ucbf	dmdl		: 1;	/* Disable manufacturer defect list */
	Ucbf	dcert		: 1;	/* Disable certification	    */
	Ucbf	serr		: 1;	/* Stop on error		    */
	Ucbf			: 3;
	Ucbf	vu		: 1;
	u_char	length[2];		/* Length of following list in bytes*/
};
#endif

struct	scsi_def_bfi {
	u_char	cyl[3];
	u_char	head;
	u_char	bfi[4];
};

struct	scsi_def_phys {
	u_char	cyl[3];
	u_char	head;
	u_char	sec[4];
};

struct	scsi_def_list {
	struct	scsi_def_header	hd;
	union {
			u_char		list_block[1][4];
		struct	scsi_def_bfi	list_bfi[1];
		struct	scsi_def_phys	list_phys[1];
	} def_list;
};

struct	scsi_format_data {
	struct scsi_format_header hd;
	union {
			u_char		list_block[1][4];
		struct	scsi_def_bfi	list_bfi[1];
		struct	scsi_def_phys	list_phys[1];
	} def_list;
};

#define	def_block	def_list.list_block
#define	def_bfi		def_list.list_bfi
#define	def_phys	def_list.list_phys

#define	SC_DEF_BLOCK	0
#define	SC_DEF_BFI	4
#define	SC_DEF_PHYS	5
#define	SC_DEF_VU	6
#define	SC_DEF_RES	7

struct	scsi_send_diag_cmd {
	u_char	cmd;
	u_char	addr[4];
	Ucbf		:8;
};

#if	defined(_BIT_FIELDS_LTOH)	/* Intel byteorder */

struct	scsi_sector_header {
	u_char	cyl[2];
	u_char	head;
	u_char	sec;
	Ucbf		:5;
	Ucbf	rp	:1;
	Ucbf	sp	:1;
	Ucbf	dt	:1;
};

#else					/* Motorola byteorder */

struct	scsi_sector_header {
	u_char	cyl[2];
	u_char	head;
	u_char	sec;
	Ucbf	dt	:1;
	Ucbf	sp	:1;
	Ucbf	rp	:1;
	Ucbf		:5;
};
#endif

#endif	/* _SCSIREG_H */
